from pathlib import Path
from configparser import ConfigParser
from collections import OrderedDict

NONE = ""
configfile = 'config.ini'

Default =  OrderedDict([('base_file_path', NONE),
                        ('query_path', NONE),
                        ('database', NONE),
                        ('notification_recipient', NONE)])

ConnectionName = OrderedDict([('rdbms', NONE),
                            ('host', NONE),
                            ('port', 0),
                            ('user', NONE),
                            ('password', NONE),
                            ('schema', NONE)])

Exchange = OrderedDict([('exchange_user', NONE),
                        ('exchange_password', NONE),
                        ('host', NONE)])

Logging = OrderedDict([('server', "FALSE"),
                        ('logging_db', NONE)])

Email = OrderedDict([('disabled', "FALSE"),
                        ('signature', NONE)])

Debug = OrderedDict([('debug_mode', "FALSE"),
                        ('debug_email', NONE)])

Admin = OrderedDict([('admin_email', NONE)])

def write_default_config():
    parser = ConfigParser()

    parser['Default'] = Default
    parser['ConnectionName'] = ConnectionName
    parser['Exchange'] = Exchange
    parser['Logging'] = Logging
    parser['Email'] = Email
    parser['Debug'] = Debug
    parser['Admin'] = Admin

    config_path = Path(configfile)
    if config_path.is_file():
        print("Config template already exists.")
    else:
        with open (configfile, 'w') as f:
            parser.write(f)
        print("New config template written as {}.".format(configfile))



from .app import config
from sqlalchemy import MetaData, ForeignKey, Table, Column, Integer, String, DateTime, Boolean, func

try:
    schema = config[config['Default']['database']]['schema']
except:
    schema = None

metadata = MetaData(schema=schema)

automated_reports = Table('automated_reports', metadata,
                Column('report_id', Integer, primary_key=True),
                Column('report_name', String(64), nullable=False),
                Column('active', Integer),
                Column('created_at', DateTime, server_default=func.now()),
                Column('updated_at', DateTime, onupdate=func.now()),
            )

automated_report_contacts = Table('automated_report_contacts', metadata,
                Column('contact_id', Integer, primary_key=True),
                Column('last_name', String(64)),
                Column('first_name', String(64)),
                Column('email_address', String(64)),
                Column('created_at', DateTime, server_default=func.now()),
                Column('updated_at', DateTime, onupdate=func.now()),
            )

automated_report_recipients = Table('automated_report_recipients', metadata,
                Column('recipient_id', Integer, primary_key=True),
                Column('report_id', Integer, ForeignKey(automated_reports.c.report_id),
                                                        nullable=False),
                Column('contact_id', Integer, ForeignKey(automated_report_contacts.c.contact_id),
                                                        nullable=False),
                Column('recipient_type', String(10), nullable=False),
                Column('created_at', DateTime, server_default=func.now()),
                Column('updated_at', DateTime, onupdate=func.now()),
            )

report_logs = Table('report_logs', metadata,
                Column('id', Integer, primary_key=True),
                Column('report_name', String(64), nullable=False),
                Column('started_at', DateTime),
                Column('completed_at', DateTime),
                Column('success', Boolean),
                Column('error_detail', String(255)),
                Column('created_at', DateTime, server_default=func.now()),
                Column('updated_at', DateTime, onupdate=func.now()),
            )




"""
RelatedRecord.py
Created by William McMonagle on 01/03/17.
Contains classes to allow for inserting, tracking, and associating of new db log records
with parent/child relationships.
"""
from sqlalchemy import Table, MetaData, ForeignKey


class RelatedRecord(object):
    """
    Represents a database record; provides insert/update functionality of
    of individual records.
    """
    def __init__(self, connection_manager, table):
        self.cm = connection_manager
        self.table = Table(table,
                            MetaData(schema=connection_manager.schema),
                            autoload=True,
                            autoload_with=connection_manager.engine)
        self.primary_key_name = self.inspect_primary_key()
        self.primary_key = None
        self.inserted = False
        self.updated = False

    def inspect_primary_key(self):
        keys = [column.name for column in self.table.columns.values() if column.primary_key]
        if len(keys) == 1:
            return keys[0]
        else:
            raise ValueError("Expected one primary key, found {}".format(len(keys)))

    def insert(self, data_to_insert):
        if not self.inserted:
            try:
                statement = self.table.insert().values(**data_to_insert)
                result = self.cm.conn.execute(statement)
                self.primary_key = result.inserted_primary_key[0]
                self.inserted = True
            except:
                raise
        else:
            raise("Cannot perform insert - record already inserted.")

    def update(self, date_to_update):
        try:
            key_column = getattr(self.table.c, self.primary_key_name)
            statement = self.table.update()\
                                    .where(key_column==self.primary_key)\
                                    .values(**date_to_update)
            self.cm.conn.execute(statement)
            self.updated = True
        except:
            raise

class ChildRecord(RelatedRecord):
    """
    A ChildRecord is the child of a RelatedRecord. Functionality is similar
    except that ChildRecord is related to the parent through a foreign key.
    If foreign key name is not provided, an attempt is made to inspect the
    reflected table.
    """
    def __init__(self, connection_manager, table, parent, foreign_key_name=None):
        super().__init__(connection_manager, table)
        self.parent = parent
        if foreign_key_name:
            self.foreign_key_name = foreign_key_name
        else:
            self.foreign_key_name = self.inspect_foreign_key()
        self.foreign_key = parent.primary_key

    def inspect_foreign_key(self):
        """
        Examines table to determine name of foreign key to parent table; validates
        name match between identified key and primary key of parent table.
        """
        parent_key = "{}.{}".format(self.parent.table.name, self.parent.primary_key_name)
        fk_cols = [column for column in self.table.columns.values() if column.foreign_keys]
        found_keys = []
        for col in fk_cols:
            found = False
            for fk in col.foreign_keys:
                if str(fk.column) == parent_key:
                    found = True
            if found:
                found_keys.append(col.name)
        if len(found_keys) == 1:
            return found_keys[0]
        else:
            raise ValueError("Expected one foreign key, found {}".format(len(found_keys)))

    def insert(self, data_to_insert):
        "Add fk key/value to dict and insert record."
        data_to_insert[self.foreign_key_name] = self.foreign_key
        super().insert(data_to_insert)

if __name__ == '__main__':
    pass



import os, re, sys, json
from decimal import Decimal
from datetime import date
from .app import config
from .connection_manager import ConnectionManager


class QueryResult(object):
    "Represent result data from an executed query. Includes capability to write results as json."

    def __init__(self, result_count=None, field_names=None, result_data=None):
        self.result_count = result_count
        self.field_names = field_names
        self.result_data = result_data

    def json_converter(self, obj):
        "Required to convert datatypes not otherwise json serializable."
        if isinstance(obj, Decimal):
            return float(obj)
        elif isinstance(obj, date):
            return obj.isoformat()
        else:
            raise TypeError("Cannot convert provided type {}".format(type(obj)))

    def as_dict(self):
        "Returns contents as list of dictionaries with headers as keys."
        if self.field_names and self.result_data:
            return [dict(zip(self.field_names, row)) for row in self.result_data]
        else:
            raise ValueError("Both field_names and result_data attributes are required to convert query results to dictionary.")

    def write_to_json(self, filename):
        contents = self.as_dict()
        with open(filename, 'w') as f:
            json.dump(contents, f, default=self.json_converter)


class QueryGenerator(object):
    "Execute SQL query and return results"
    def __init__(self, cm, filename=None, params=None, sql=None):
        self.cm = cm
        self.filename = filename
        self.params = params
        self.sql = sql

    def construct_query(self):
        # Resolve full SQL statement.
        if self.filename:
            reader = QueryReader(filename=self.filename, params=self.params)
            self.sql = reader.sql
        else:
            raise Exception("Cannot construct query without providing the SQL filename.")

    def execute(self):

        if self.sql is None:
            self.construct_query()

        try:
            result_proxy = self.cm.conn.execute(self.sql)
            result_data = result_proxy.fetchall()
        except:
            print(sys.exc_info())
            raise RuntimeError("Unable to execute query.")

        query_results = QueryResult(result_count=len(result_data),
                                    field_names=result_proxy.keys(),
                                    result_data=result_data)
        return query_results


class QueryReader(object):
    """
    QueryReader is used to read, and optionally to parameterize, .sql files.

    Saved queries requiring paremeters at runtime should use placeholders in the format
    matching the raw_pattern attribute, which is by default:
        #{parameter_name} e.g. #{first_name}, #{last_name}, etc.

    Replacement values must be provided for all parameter placeholders.

    Simple Usage - No Parameters

    >> my_query = QueryReader(filename='my_query')
    >> my_query.sql
    select * from table;

    Usage with Parameters

    >> my_query = QueryReader(filename='my_query', params={'field': 'value'})
    >> my_query.sql
    select * from table where field = 'value';

    """
    def __init__(self, filename=None, params=None):
        self.filename = filename
        self.params = params
        self.raw_sql = None
        self.sql = None
        self.query_path = config['Default']['query_path']
        self.raw_pattern = '(#{[a-z]*})'
        if filename:
            self.read()
            self.find_values_to_replace()
            if self.to_replace:
                self.replace_params()
            self.validate()

    def __repr__(self):
        if self.sql:
            return self.sql
        elif self.raw_sql:
            return self.raw_sql
        else:
            return "< QueryReader object >"

    def read(self):
        "Reads and stores query contents"
        file_path = os.path.join(self.query_path, self.filename + '.sql')
        with open(file_path, 'r') as f:
            self.raw_sql = f.read()
            self.sql = self.raw_sql

    def find_values_to_replace(self):
        "Use pattern to identify all parameters in raw sql which need to be replaced."
        regexp = re.compile(self.raw_pattern)
        self.to_replace = regexp.findall(self.raw_sql)

    def replace_params(self):
        "For every param, find and replace placeholder with appropriate value."
        raw_sql = self.raw_sql
        for placeholder in self.to_replace:
            newreg = re.compile(placeholder)
            repl = self.get_replacement_value(placeholder)
            if repl:
                raw_sql = newreg.sub(str(repl), raw_sql)
        self.sql = raw_sql

    def get_replacement_value(self, to_be_replaced):
        "Given placeholder to be replaced, get name of parameter from within the pattern and lookup parameter value."
        name_reg = re.compile('[a-z]+')
        param_name = name_reg.search(to_be_replaced).group()
        return self.params.get(param_name)

    def validate(self):
        "Check whether any placeholders have not been provided a replacement value."
        regexp = re.compile(self.raw_pattern)
        missing = regexp.findall(self.sql)
        if missing:
            raise NameError("Value not provided for placeholder {}".format(missing))


if __name__ == '__main__':
    pass



import pymysql
from sqlalchemy import create_engine
from .app import config


class ConnectionManager():
    def __init__(self, db=None):
        self.db = db
        self.config = config
        if db:
            self.unpack_params()

    def unpack_params(self):
        self.rdbms = self.config[self.db].get('rdbms')
        self.db_host = self.config[self.db].get('host')
        self.db_port = int(self.config[self.db].get('port', 0))
        self.db_user = self.config[self.db].get('user')
        self.db_password = self.config[self.db].get('password')
        self.schema = self.config[self.db].get('schema')

    def connect(self):
        if not self.db:
            raise ValueError("Cannot connect - db attribute not set.")
        try:
            self.engine = self.create_engine()
            self.conn = self.engine.connect()
        except ValueError:
            raise
        except:
            raise RuntimeError("Unable to connect to database {}".format(self.db))

    def create_engine(self):
        if self.rdbms == 'sqlite':
            return create_engine('sqlite:///{db_host}'.format(**self.__dict__))
        elif self.rdbms == 'mysql':
            return create_engine('mysql+pymysql://{db_user}:{db_password}@{db_host}'.format(**self.__dict__))
        else:
            raise ValueError("Unsupported RDBMS: {}".format(self.rdbms))

    def close(self):
        self.conn.close()
        self.engine.dispose()


if __name__ == '__main__':
    pass



#!/usr/bin/env python
# encoding: utf-8
"""
GenericReport.py
Created on 01/26/17.
This module contains functionality to simplify the creation of new automated reports.
Features include creation of Excel worksheets from query results, emailing of results,
and database logging.
"""

import sys
import os.path
from sqlalchemy import select
from .app import config
from .models import (metadata,
                    automated_reports,
                    automated_report_contacts,
                    automated_report_recipients,
                    report_logs)
from .connection_manager import ConnectionManager
from .related_record import RelatedRecord
from .mailer import Mailer
from .queries import QueryGenerator, QueryReader
from .xlsx import WorkbookBuilder
from . import TimeHelper


class GenericReport():
    """A generic report object to be used to facilitate
    easy setup and configuration of new automated reports.

    Keyword arguments
    :report_title: Used in the filename of the resulting report.
    :report_name: Uniquely identifies a report. Used for recipient lookup
        and logging.
    :logging_enabled: (Optional) Default value of True. If logging is enabled,
        the report will attempt to create a log record each time it is instantiated.
        Only disable if the report needs to be run without connection to the standard
        reporting database.

    Here is an example of sample usage:

    report = GenericReport(
                            report_name='test_report',
                            report_title='Test Report'
                            )
    report.build_file()
    report.create_worksheet_from_query(
                                        query={'name': 'test_query'},
                                        sheet_name='Sheet1'
                                        )
    report.subject = "Today's Test Report"
    report.message = "Attached is the latest report"
    report.execute()

    Notes on sample code:
    (1) A report is not required to have an attachment,
        but an attachment is assumed.
    (2) An Excel attachment can contain any number
        of worksheets. Simply call create_worksheet_from_query
        with the appropriate query parameters and worksheet
        name, as many times as you like. Worksheet names
        must be valid and unique.
    (3) After providing a subject and message body,
        the execute method will take care of the rest,
        including email circulation, error handling,
        and notification.

    Notes on database configuration:
    (1) Reports must have a record in the automated_reports table,
        including report_name and active flag (1 or 0).
    (2) Reports also must have at least one recipient
        in the automated_report_recipients table, and at least
        one recipient must be designed with a recipient type of
        'to'. Each report can have one or more recipients with
        a recipient type of 'cc'.
    (3) Report recipient records are associated with:
        (a) automated_reports through report_id
        (b) automated_report_contacts through contact_id
    (4) If you need a recipient who is not in the
        automated_report_contacts table, add them to it.

    """
    def __init__(self, report_title, report_name, logging_enabled=True):
        # -----------------------------------------
        # Assign arguments to instance attributes.
        # -----------------------------------------
        self.report_title = report_title
        self.report_name = report_name
        self.logging_enabled = logging_enabled
        # -----------------------------------------
        # Setup configuration variables and defaults.
        # By default, reports will be sent even if there are no results.
        # This behavior can be overridden for individual reports. Record count
        # initialized at 0, but incremented by execute_query.
        # -----------------------------------------
        self.send_if_blank = True
        self.record_count = 0
        self.workbook = None
        self.attachments = []
        self.error_detail = []
        self.to_recipients = []
        self.cc_recipients = []
        self.email_sent = False
        self.file_path = config['Default'].get('base_file_path')
        self.default_db = config['Default'].get('database')
        self.notification_recipient = config['Default'].get('notification_recipient')
        self.default_cm = ConnectionManager(self.default_db)
        self.default_cm.connect()
        self.active = self.check_if_active()
        self.create_log_record()

    def __del__(self):
        try:
            self.default_cm.close()
        except:
            pass

    def log_error_detail(self, error_context):
        """Provided a description of when error was raised, assembles error message for log
        and appends the message."""
        error_value = sys.exc_info()[1]
        error_message = error_context + '; ' + str(error_value) + ': ' + str(error_value.__doc__)
        self.error_detail.append(error_message)

    def check_if_active(self):
        """Queries database to determine if the 'active' attribute for this report
        is set to 1 (active) or 0 (inactive).
        """
        statement = select([automated_reports.c.active])\
                        .where(automated_reports.c.report_name==self.report_name)
        q = QueryGenerator(cm=self.default_cm, sql=statement)
        try:
            results = q.execute()
            if results.result_data[0][0] == 1:
                return True
            else:
                return False
        except IndexError:
            self.log_error_detail("Report does not exist.")
            raise Exception("Report does not exist.")

    def create_log_record(self):
        """Inserts new log record into default report logging table
        using RelatedRecord and saves instance of RelatedRecord for updating
        later on with results of report execution.
        """
        if self.active and self.logging_enabled:
            report_log = RelatedRecord(self.default_cm, 'report_logs')
            report_log.insert({'report_name': self.report_name,
                                'started_at': TimeHelper.now(string=False)})
            self.report_log = report_log

    def finalize_log_record(self):
        "Update log at conclusion of report execution to indicate success/failure."
        if self.logging_enabled:
            if self.error_detail:
                errors = "; ".join(self.error_detail).replace("'", "")
                data_to_update = {'completed_at': TimeHelper.now(string=False),
                                    'success': 0,
                                    'error_detail': errors}
            else:
                data_to_update = {'completed_at': TimeHelper.now(string=False),
                                    'success': 1}
        self.report_log.update(data_to_update)

    def build_file(self):
        """Loads file and path information and creates XLSX Writer Workbook object.
        Default file is named with convention yyyy-mm-dd - Report Name.xlsx
        """
        # Construct the file path.
        try:
            report_file_name = "{} - {}.xlsx".format(TimeHelper.today(), self.report_title)
            self.report_file = os.path.join(self.file_path, report_file_name)
            # Create the workbook.
            self.workbook_builder = WorkbookBuilder(filename=self.report_file)
        except:
            self.log_error_detail("Unable to build file")

    def close_workbook(self):
        """Closes workbook object after creation is complete.
        Should always be executed before sending file."""
        if self.workbook_builder:
            self.workbook_builder.workbook.close()
        if self.report_file:
            self.attachments.append(self.report_file)

    def execute_query(self, sql, cm=None, increment_counter=True):
        """
        Executes SQL and returns QueryResult object,
        containing data and metadata.
        """
        if cm is None:
            cm = self.default_cm
        q = QueryGenerator(cm=cm, sql=sql)
        try:
            results = q.execute()
            if increment_counter:
                self.record_count += results.result_count
            return results
        except:
            self.log_error_detail("Unable to execute query.")

    def make_worksheet(self, sheet_name, query_results):
        "Adds worksheet to workbook using provided query results."
        try:
            self.workbook_builder.add_worksheet(sheet_name=sheet_name,
                                                field_names=query_results.field_names,
                                                sheet_data=query_results.result_data
                                            )
        except:
            self.log_error_detail("Unable to add worksheet {}".format(sheet_name))

    def read_query(self, query):
        """
        Uses QueryReader to read .sql file and parameterize query as necessary.

        Args:
            query (dict):   Should contain filename (str) and params (dict, optional).
        """
        filename = query.get('filename')
        params = query.get('params')
        reader = QueryReader(filename=filename, params=params)
        return reader

    def create_worksheet_from_query(self, query, sheet_name, read_required=False, cm=None):
        """
        Args:
            query           (str,
                            dict, or
                            sqlalchemy.sql.selectable.Select): A SQL query ready
                                for execution, or a dict containing information
                                necessary to read and/or parameterize a query.
            sheet_name      (str): The name of the resulting worksheet.
            read_required   (bool): Defaults to False. Set to true if passing a
                                query which must be read and/or parameterized
                                from file. If true, query parameter should be
                                a dict.
            cm              (ConnectionManager): Defaults to None, in which case
                                default ConnectionManager is used.

        """
        if cm is None:
            cm = self.default_cm
        if read_required:
            reader = self.read_query(query)
            sql = reader.sql
        else:
            sql = query
        results = self.execute_query(cm=cm, sql=sql)
        self.make_worksheet(sheet_name=sheet_name, query_results=results)

    def get_recipients(self):
        "Performs lookup in database for report recipients based on report name."
        statement = select([automated_report_contacts.c.email_address,
                            automated_report_recipients.c.recipient_type])\
                        .select_from(automated_reports\
                        .join(automated_report_recipients)\
                        .join(automated_report_contacts))\
                        .where(automated_reports.c.report_name==self.report_name)
        results = self.execute_query(statement, increment_counter=False)

        for recipient in results.result_data:
            if recipient.recipient_type == 'to':
                self.to_recipients.append(recipient.email_address)
            else:
                self.cc_recipients.append(recipient.email_address)

        if not self.to_recipients:
            raise KeyError("No primary recipient found for the provided report.")

    def build_email(self):
        "Instantiates Mailer object using provided parameters."
        if not self.to_recipients:
            self.get_recipients()
        email = Mailer()
        email.recipients = self.to_recipients
        email.cc_recipients = self.cc_recipients
        email.subject = self.subject
        email.message = self.message
        email.attachments = self.attachments
        self.email = email

    def check_whether_to_send_email(self):
        "Determines whether email should be sent based given errors, settings, and result count."
        if self.error_detail:
            return False
        elif not self.send_if_blank and self.record_count == 0:
            return False
        else:
            return True

    def send_email(self):
        "Executes the send_email method of the Mailer."
        try:
            self.email.send_email()
            self.email_sent = True
        except:
            self.log_error_detail("Unable to send email")

    def build_and_send_email(self):
        """If email should be sent, builds email object and sends email."""
        if self.check_whether_to_send_email():
            self.build_email()
            self.send_email()

    def send_failure_notification(self):
        "Sends failure notification to identified recipients, including logged errors."

        message = """Execution of the following report failed: {}

The following errors were logged:
""".format(self.report_title)

        for i, error in enumerate(self.error_detail, 1):
            message += str(i) + '. ' + error + '\n'

        email = Mailer()
        email.recipients = [self.notification_recipient]
        email.cc_recipients = ['']
        email.subject = "Report Failure: {}".format(self.report_title)
        email.message = message
        email.attachments = []
        email.send_email()
        self.failure_notification_sent = True

    def execute(self):
        """
        After instantiation and setup, and after
        any query results have been added to the workbook,
        this method executes and finalizes the report,
        ensures errors are handled, and performs cleanup.
        """
        self.close_workbook()
        if self.active:
            self.build_and_send_email()
            self.finalize_log_record()
        self.default_cm.close()
        if self.error_detail:
            self.send_failure_notification()

if __name__ == '__main__':
    pass



import xlsxwriter

class WorkbookBuilder(object):
    """
    Allows for creation of Excel workbooks containing arbitrary worksheets
    using the xlsxwriter library.
    """
    def __init__(self, filename=None):
        self.filename = filename
        self.header_params = {'bold': True}
        self.workbook_options = {'constant_memory': True,
                                'default_date_format': 'mm/dd/yy'}
        if filename:
            self.create_workbook()

    def create_workbook(self):
        "Given filename, workbook options, and head format, create workbook."
        workbook = xlsxwriter.Workbook(self.filename, self.workbook_options)
        self.header_format = workbook.add_format(self.header_params)
        self.workbook = workbook

    def close_workbook(self):
        "An exception may be raised if workbook is not closed explicitly."
        self.workbook.close()

    def add_worksheet(self, sheet_name, field_names, sheet_data, row_start=0, col_start=0):
        worksheet = self.workbook.add_worksheet(sheet_name)

        e_row = row_start
        e_col = col_start

        # Write field names in first row.
        for i, field_name in enumerate(field_names):
            worksheet.write(e_row,
                            e_col + i,
                            field_name,
                            self.header_format)

        # Increment row counter for incoming data.
        e_row += 1

        # Write the data.
        for row in sheet_data:
            for x in range(0, len(row)):
                worksheet.write(e_row, e_col + x, row[x])
            e_row += 1





class SimpleWorkflow():
    """
    :functions: A list of functions to be executed:
        {'function': function_object, 'args': {'arg_name': arg_value}}

    Use to execute a sequence of functions. Assumes that each function
    returns boolean. Execution will continue, in order,
    unless/until False is returned, at which point execution will cease.

    Future implementation - option to stop on error vs. continue on error.
    """
    def __init__(self, functions):
        self.functions = functions
        self.completed = None
        self.failed_on = None

    def run(self):
        while self.functions:
            # pop FIRST function from list
            function = self.functions.pop(0)
            try:
                # execute function by unpacking dictionary
                success = function['function'](**function['args'])
            except:
                success = False
            if not success:
                self.completed = False
                self.failed_on = function
                break
        else: # only execute if loop finishes without breaking
            self.completed = True


if __name__ == '__main__':
    pass



"""
Mailer.py

"""

import sys
import os
import smtplib
from email import encoders
from email.mime.base import MIMEBase
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from .app import config

COMMASPACE = ', '


class Mailer:
    def __init__(self, **kwargs):
        self.properties = kwargs
        self.debug_mode = config.getboolean('Debug', 'debug_mode')
        self.debug_email = [config['Debug']['debug_email']]
        self.send_from = config['Exchange']['exchange_user']
        self.exchange_user = config['Exchange']['exchange_user']
        self.exchange_password = config['Exchange']['exchange_password']
        self.host = config['Exchange'].get('host', 'smtp.office365.com')
        self.signature = config['Email']['signature']
        self.admin_email = config['Admin']['admin_email']

    def send_email(self):
        if config['Email']['disabled'] == 'True':
            print("Email disabled - exiting mailer.")
            exit()

        # Create the enclosing (outer) message
        outer = MIMEMultipart('alternative')
        outer['Subject'] = self.subject
        outer['To'] = COMMASPACE.join(self.recipients)
        outer['CC'] = COMMASPACE.join(self.cc_recipients)
        outer['From'] = self.send_from

        msg = MIMEBase('application', "octet-stream")

        # Add the text of the email
        email_body = MIMEText(self.message, 'plain')
        outer.attach(email_body)

        # Add the attachments
        if self.attachments:
            for file in self.attachments:
                try:
                    with open(file, 'rb') as fp:
                        msg.set_payload(fp.read())
                    encoders.encode_base64(msg)
                    msg.add_header('Content-Disposition',
                                   'attachment',
                                   filename=os.path.basename(file))
                    outer.attach(msg)
                except:
                    print("Unable to add the attachment to the email")
                    raise

        composed = outer.as_string()

        all_recipients = set(self.recipients + self.cc_recipients)
        all_recipients = list(filter(None, all_recipients))

        try:
            with smtplib.SMTP(host='smtp.office365.com', port='587') as s:
                s.ehlo()
                s.starttls()
                s.ehlo()
                s.login(self.exchange_user, self.exchange_password)
                s.sendmail(self.exchange_user,
                           all_recipients,
                           composed)
                s.close()
                print("Email sent")
        except smtplib.SMTPConnectError as err:
            print("Unable to connect to the SMTP server to send email: {}".format(err))
            raise
        except:
            print("Unable to send email: {}".format(sys.exc_info()[0]))
            raise

if __name__ == '__main__':
    pass



from setuptools import setup

setup(name='porthole',
      version='0.3',
      description='An automated reporting package.',
      author='Billy McMonagle',
      author_email='speedyturkey@gmail.com',
      url='https://github.com/speedyturkey/porthole',
      packages=['porthole'],
      install_requires=[
                        'pymysql',
                        'xlsxwriter',
                        'SQLAlchemy'
                    ],
      zip_safe=False)



from .app import config
from .setup_config import write_default_config
from .connection_manager import ConnectionManager
from .queries import QueryGenerator, QueryReader, QueryResult
from .xlsx import WorkbookBuilder
from .mailer import Mailer
from .workflows import SimpleWorkflow
from .related_record import RelatedRecord, ChildRecord
from .reports import GenericReport
